@@ -7,7 +7,7 @@ import os
 import fnmatch
 from pathlib import Path
 import requests
-from pyutilb import log, ocr_youdao
+from pyutilb import log, YamlBoot, BreakException, ocr_youdao
 from pyutilb.util import *
 import base64
 from AppiumBoot.validator import Validator
@@ -36,35 +36,16 @@ def get_text_or_content(self):
     return self.get_attribute("content-desc")
 WebElement.get_text_or_content = get_text_or_content
 
-# 跳出循环的异常
-class BreakException(Exception):
-    def __init__(self, condition):
-        self.condition = condition # 跳转条件
 
 # appium基于yaml的启动器
-class Boot(object):
+class Boot(YamlBoot):
 
     def __init__(self):
-        # 延迟初始化driver
-        self.driver = None
-        # 当前页面的校验器, 依赖于driver
-        self.validator = None
-        # 当前页面的提取器, 依赖于driver
-        self.extractor = None
-        # 延迟初始化的app包
-        self.package = None
-        # 步骤文件所在的目录
-        self.step_dir = None
-        # 已下载过的url对应的文件，key是url，value是文件
-        self.downloaded_files = {}
-        # 基础url
-        self._base_url = None
+        super().__init__()
         # 动作映射函数
-        self.actions = {
+        actions = {
             'init_driver': self.init_driver,
             'close_driver': self.close_driver,
-            'sleep': self.sleep,
-            'print': self.print,
             'input_by_id': self.input_by_id,
             'input_by_aid': self.input_by_aid,
             'input_by_class': self.input_by_class,
@@ -105,15 +86,11 @@ class Boot(object):
             'print_performance': self.print_performance,
             'start_recording_screen': self.start_recording_screen,
             'stop_recording_screen': self.stop_recording_screen,
-            'for': self.do_for,
-            'once': self.once,
-            'break_if': self.break_if,
-            'moveon_if': self.moveon_if,
+            'alert_accept': self.alert_accept,
+            'alert_dismiss': self.alert_dismiss,
             'moveon_if_exist_by': self.moveon_if_exist_by,
+            'break_if_exist_by': self.break_if_exist_by,
             'break_if_not_exist_by': self.break_if_not_exist_by,
-            'include': self.include,
-            'set_vars': self.set_vars,
-            'print_vars': self.print_vars,
             'base_url': self.base_url,
             'get': self.get,
             'post': self.post,
@@ -135,71 +112,24 @@ class Boot(object):
             'extract_by_class': self.extract_by_class,
             'extract_by_eval': self.extract_by_eval,
         }
-        set_var('boot', self)
+        self.add_actions(actions)
+
+        # 延迟初始化driver
+        self.driver = None
+        # 当前页面的校验器, 依赖于driver
+        self.validator = None
+        # 当前页面的提取器, 依赖于driver
+        self.extractor = None
+        # 延迟初始化的app包
+        self.package = None
+        # 已下载过的url对应的文件，key是url，value是文件
+        self.downloaded_files = {}
+        # 基础url
+        self._base_url = None
         # 是否在录屏
         self.recording = False
-        # 当前文件
-        self.step_file = None
 
-    '''
-    执行入口
-    :param step_files 步骤配置文件或目录的列表
-    '''
-    def run(self, step_files):
-        for path in step_files:
-            # 1 模式文件
-            if '*' in path:
-                dir, pattern = path.rsplit(os.sep, 1)  # 从后面分割，分割为目录+模式
-                if not os.path.exists(dir):
-                    raise Exception(f'步骤配置目录不存在: {dir}')
-                self.run_1dir(dir, pattern)
-                return
-
-            # 2 不存在
-            if not os.path.exists(path):
-                raise Exception(f'步骤配置文件或目录不存在: {path}')
-
-            # 3 目录: 遍历执行子文件
-            if os.path.isdir(path):
-                self.run_1dir(path)
-                return
-
-            # 4 纯文件
-            self.run_1file(path)
-
-    # 执行单个步骤目录: 遍历执行子文件
-    # :param path 目录
-    # :param pattern 文件名模式
-    def run_1dir(self, dir, pattern ='*.yml'):
-        # 遍历目录: https://blog.csdn.net/allway2/article/details/124176562
-        files = os.listdir(dir)
-        files.sort() # 按文件名排序
-        for file in files:
-            if fnmatch.fnmatch(file, pattern): # 匹配文件名模式
-                file = os.path.join(dir, file)
-                if os.path.isfile(file):
-                    self.run_1file(file)
-
-    # 执行单个步骤文件
-    # :param step_file 步骤配置文件路径
-    # :param include 是否inlude动作触发
-    def run_1file(self, step_file, include = False):
-        # 获得步骤文件的绝对路径
-        if include: # 补上绝对路径
-            if not os.path.isabs(step_file):
-                step_file = self.step_dir + os.sep + step_file
-        else: # 记录目录
-            step_file = os.path.abspath(step_file)
-            self.step_dir = os.path.dirname(step_file)
-
-        log.debug(f"加载并执行步骤文件: {step_file}")
-        # 获得步骤
-        steps = read_yaml(step_file)
-        self.step_file = step_file
-        # 执行多个步骤
-        self.run_steps(steps)
-
-    # 执行多个步骤
+    # 执行多个步骤: 重写以便异常时结束录屏
     def run_steps(self, steps):
         try:
             # 逐个步骤调用多个动作
@@ -221,7 +151,7 @@ class Boot(object):
         for step in steps:
             for action, param in step.items():
                 if i > start and action == 'stop_recording_screen':
-                    log.error('发生异常, 结束录屏')
+                    log.error('Exception occur, stop recording screen')
                     # 调用动作
                     # self.run_action(action, param)
                     self.stop_recording_screen(param)
@@ -229,25 +159,6 @@ class Boot(object):
 
                 i += 1
 
-    '''
-    执行单个动作：就是调用动作名对应的函数
-    :param action 动作名
-    :param param 参数
-    '''
-    def run_action(self, action, param):
-        if 'for(' in action:
-            n = int(action[4:-1])
-            self.do_for(param, n)
-            return
-
-        if action not in self.actions:
-            raise Exception(f'无效动作: [{action}]')
-
-        # 调用动作对应的函数
-        log.debug(f"处理动作: {action}={param}")
-        func = self.actions[action]
-        func(param)
-
     # --------- 动作处理的函数 --------
     # 初始化driver
     def init_driver(self, config):
@@ -265,15 +176,19 @@ class Boot(object):
             }
         }
         '''
+        # 替换配置中的变量
+        config = replace_var(config, False)
         if 'executor' in config:
             executor = config['executor']
         else:
             executor = 'http://localhost:4723/wd/hub'
         if 'desired_caps' not in config:
-            raise Exception(f'未指定driver的desired_caps参数')
+            raise Exception(f'Miss driver desired_caps parameter')
         desired_caps = config['desired_caps']
         if 'appPackage' in desired_caps:
             package = desired_caps['appPackage']
+        # 记录ios/android平台
+        self.platform = desired_caps['platformName'].lower()
         # driver
         self.driver = webdriver.Remote(executor, desired_caps)
         # 当前页面的校验器, 依赖于driver
@@ -288,40 +203,13 @@ class Boot(object):
             self.driver = None
             self.package = None
 
-    # for循环
-    # :param steps 每个迭代中要执行的步骤
-    # :param n 循环次数
-    def do_for(self, steps, n = None):
-        label = f"for({n})"
-        if n == None:
-            n = sys.maxsize # 最大int，等于无限循环次数
-            label = f"for(∞)"
-        log.debug(f"-- 开始循环: {label} -- ")
-        try:
-            for i in range(n):
-                # i+1表示迭代次数比较容易理解
-                log.debug(f"第{i+1}次迭代")
-                set_var('for_i', i+1)
-                self.run_steps(steps)
-        except BreakException as e:  # 跳出循环
-            log.debug(f"-- 跳出循环: {label}, 跳出条件: {e.condition} -- ")
-        else:
-            log.debug(f"-- 终点循环: {label} -- ")
+    @property
+    def is_android(self):
+        return self.platform == 'android'
 
-    # 执行一次子步骤，相当于 for(1)
-    def once(self, steps):
-        self.do_for(steps, 1)
-
-    # 检查并继续for循环
-    def moveon_if(self, expr):
-        # break_if(条件取反)
-        self.break_if(f"not ({expr})")
-
-    # 跳出for循环
-    def break_if(self, expr):
-        val = eval(expr, globals(), bvars)  # 丢失本地与全局变量, 如引用不了json模块
-        if bool(val):
-            raise BreakException(expr)
+    @property
+    def is_ios(self):
+        return self.platform == 'ios'
 
     # 检查并继续for循环
     def moveon_if_exist_by(self, config):
@@ -332,29 +220,10 @@ class Boot(object):
         if not self.exist_by_any(config):
             raise BreakException(config)
 
-    # 加载并执行其他步骤文件
-    def include(self, step_file):
-        self.run_1file(step_file, True)
-
-    # 设置变量
-    def set_vars(self, vars):
-        for k, v in vars.items():
-            v = replace_var(v)  # 替换变量
-            set_var(k, v)
-
-    # 打印变量
-    def print_vars(self, _):
-        log.info(f"打印变量: {bvars}")
-
-    # 睡眠
-    def sleep(self, seconds):
-        seconds = replace_var(seconds)  # 替换变量
-        time.sleep(int(seconds))
-
-    # 打印
-    def print(self, msg):
-        msg = replace_var(msg)  # 替换变量
-        log.info(msg)
+    # 跳出for循环
+    def break_if_exist_by(self, config):
+        if self.exist_by_any(config):
+            raise BreakException(config)
 
     # 解析响应
     def _analyze_response(self, res, config):
@@ -392,16 +261,16 @@ class Boot(object):
     # :param input_data 表单数据, key是输入框的路径, value是填入的值
     def _input_by_type_and_data(self, type, input_data):
         for name, value in input_data.items():
-            log.debug(f"替换变量： {name} = {value}")
             value = replace_var(value)  # 替换变量
 
             # 找到输入框
             try:
                 ele = self.find_by(type, name)
             except Exception as ex:  # 找不到元素
-                log.error(f"找不到输入元素{name}", exc_info = ex)
+                log.error(f"Input element not found{name}", exc_info = ex)
                 continue
 
+            # ele.set_value(value)
             ele.clear() # 先清空
             ele.send_keys(value) # 后输入
 
@@ -422,7 +291,7 @@ class Boot(object):
                 if type == 'xpath': # xpath支持变量
                     path = replace_var(path)
                 return self.driver.find_element(type2by(type), path)
-        raise Exception(f"没有查找类型: {config}")
+        raise Exception(f"Invalid find type: {config}")
 
     # 根据任一类型，查找元素
     def find_all_by_any(self, config):
@@ -431,7 +300,7 @@ class Boot(object):
             if type in config:
                 path = config[type]
                 return self.driver.find_elements(type2by(type), path)
-        raise Exception(f"没有查找类型: {config}")
+        raise Exception(f"Invalid find type: {config}")
 
     # 根据指定类型，检查元素是否存在
     def exist_by(self, type, path):
@@ -449,14 +318,14 @@ class Boot(object):
         except NoSuchElementException:
             return False
 
-    # 屏幕滑动(传坐标)
+    # 屏幕滑动(传坐标), 无duration
     # :param config {from, to}
     def flick(self, config):
         x1, y1 = config['from'].split(",", 1) # 起点位置
         x2, y2 = config['to'].split(",", 1) # 终点位置
         self.driver.flick(x1, y1, x2, y2)
 
-    # 屏幕横扫(传坐标) -- 多次调用多次滑,就算参数一样
+    # 屏幕横扫(传坐标), 有duration -- 多次调用多次滑,就算参数一样
     # :param config {from, to, duration}
     def swipe(self, config):
         x1, y1 = config['from'].split(",", 1) # 起点位置
@@ -533,6 +402,16 @@ class Boot(object):
         duration = 0.1 * 1000
         self.driver.swipe(x1, ym, x2, ym, duration)
 
+    # 向下滚, 仅ios有效
+    def scroll_down_by(self, config):
+        ele = self.find_by_any(config)
+        self.driver.execute_script("mobile: scroll", {"direction": 'down', 'elementid': ele.id})
+
+    # 向上滚, 仅ios有效
+    def scroll_up_by(self, config):
+        ele = self.find_by_any(config)
+        self.driver.execute_script("mobile: scroll", {"direction": 'up', 'elementid': ele.id})
+
     # 移动轨迹(传坐标序列)
     # :param positions 坐标序列 如x1,y1;x2,y2
     def move_track(self, positions):
@@ -788,7 +667,7 @@ class Boot(object):
             _from = replace_var(config['from'])
             content = read_file(_from)
         else:
-            raise Exception('未指定 content 或 from, 无法获得写入的内容')
+            raise Exception('Miss content or from parameter, cannot get push content')
         # 转base64
         data = bytes(content, 'utf-8')
         data = base64.b64encode(data).decode('utf-8')
@@ -846,6 +725,18 @@ class Boot(object):
         path = self._prepare_save_file({}, path)
         write_byte_file(path, data)
 
+    # 点击弹框的确定按钮, 如授权弹框的允许
+    def alert_accept(self, _):
+        src = self.driver.page_source
+        if '允许' in src or 'allow' in src:
+            self.driver.switch_to.alert.accept()
+
+    # 取消弹框, 如授权弹窗的禁止
+    def alert_dismiss(self, _):
+        src = self.driver.page_source
+        if '允许' in src or 'allow' in src:
+            self.driver.switch_to.alert.dismiss()
+
     # 设置基础url
     def base_url(self, url):
         self._base_url = url
@@ -940,7 +831,7 @@ class Boot(object):
                     newname = f"{save_file}-{i}"
                 if not os.path.exists(newname):
                     return newname
-            raise Exception('目录太多文件，建议新建目录')
+            raise Exception('Too many file in save_dir, please change other directory.')
 
         return save_file
 
@@ -956,7 +847,7 @@ class Boot(object):
         # 设置变量
         set_var('download_file', save_file)
         self.downloaded_files[url] = save_file
-        log.debug(f"下载文件: url为{url}, 另存为{save_file}")
+        log.debug(f"Dowload file: url is {url}, save path is{save_file}")
         return save_file
 
     # 识别url中的验证码
@@ -982,7 +873,7 @@ class Boot(object):
         captcha = ocr_youdao.recognize_text(file_path)
         # 设置变量
         set_var('captcha', captcha)
-        log.debug(f"识别验证码: 图片为{file_path}, 验证码为{captcha}")
+        log.debug(f"Recognize captcha: image file is {file_path}, captcha is {captcha}")
         # 删除文件
         #os.remove(file)
 
@@ -1025,17 +916,18 @@ class Boot(object):
     def extract_by_eval(self, fields):
         return self.extractor.run_eval(fields)
 
-
 # cli入口
 def main():
     # 基于yaml的执行器
     boot = Boot()
+    # 读元数据：author/version/description
+    dir = os.path.dirname(__file__)
+    meta = read_init_file_meta(dir + os.sep + '__init__.py')
+    # 步骤配置的yaml
+    step_files = parse_cmd('AppiumBoot', meta['version'])
+    if len(step_files) == 0:
+        raise Exception("Miss step config file or directory")
     try:
-        # 步骤配置的yaml
-        if len(sys.argv) > 1:
-            step_files = sys.argv[1:]
-        else:
-            raise Exception("未指定步骤配置文件或目录")
         # 执行yaml配置的步骤
         boot.run(step_files)
     except Exception as ex:
@@ -1046,8 +938,8 @@ def main():
             src = boot.driver.page_source
             # report_to_sauce(boot.driver.session_id)
             # take_screenshot_and_logcat(boot.driver, device_logger, calling_request)
-        # log.error(f"异常环境:当前步骤文件为 {step_file}, 当前activity为 {activity}, 当前层级为 {src}", exc_info = ex)
-        log.error(f"异常环境:当前步骤文件为 {boot.step_file}, 当前activity为 {activity}", exc_info = ex)
+        # log.error(f"Exception occurs: current step file is {step_file}, current activity is {activity}, current page source is {src}", exc_info = ex)
+        log.error(f"Exception occurs: current step file is {boot.step_file}, current activity is {activity}", exc_info = ex)
         raise ex
     finally:
         boot.close_driver()
